@@ -3,7 +3,7 @@ package Test::XML::Simple;
 use strict;
 use warnings;
 
-our $VERSION = '0.10';
+our $VERSION = '1.02';
 
 use Test::Builder;
 use Test::More;
@@ -12,7 +12,6 @@ use XML::LibXML;
 
 my $Test = Test::Builder->new();
 my $Xml;
-my $last_xml_string = "";
 
 sub import {
    my $self = shift;
@@ -41,20 +40,27 @@ sub xml_valid($;$) {
 
 sub _valid_xml {
   my $xml = shift;
-  return $Xml if defined($xml) and $xml eq $last_xml_string;
-
+ 
+  local $Test::Builder::Level = $Test::Builder::Level + 2; 
   return fail("XML is not defined") unless defined $xml;
   return fail("XML is missing")     unless $xml;
-  return fail("string can't contain XML: no tags") 
-    unless ($xml =~ /</ and $xml =~/>/);
-  eval {$Xml = XML::LibXML->new->parse_string($xml)};
-  $@ ? return fail($@)
-     : return $Xml;
+  if ( ref $xml ) {
+      return fail("accept only 'XML::LibXML::Document' as object") unless ref $xml eq 'XML::LibXML::Document';
+      $Xml = $xml;
+  }
+  else {
+    return fail("string can't contain XML: no tags") 
+      unless ($xml =~ /</ and $xml =~/>/);
+    eval { $Xml = XML::LibXML->new->parse_string($xml); };
+    do { chomp $@; return fail($@) } if $@;
+  }
+  return $Xml;
 }
 
 sub _find {
   my ($xml_xpath, $xpath) = @_;
   my @nodeset = $xml_xpath->findnodes($xpath);
+  local $Test::Builder::Level = $Test::Builder::Level + 2; 
   return fail("Couldn't find $xpath") unless @nodeset;
   wantarray ? @nodeset : \@nodeset;
 }
@@ -84,6 +90,7 @@ sub xml_is_long($$$;$) {
 sub _xml_is {
   my ($comp_sub, $xml, $xpath, $value, $comment) = @_;
 
+  local $Test::Builder::Level = $Test::Builder::Level + 2; 
   my $parsed_xml = _valid_xml($xml);
   return 0 unless $parsed_xml;
 
@@ -161,12 +168,14 @@ sub _xml_like {
       }
       if (! $found) {
         $comment = "(no comment)" unless defined $comment;
-       return ok(0, "$comment - no match in tag contents (including CDATA)");
+	local $Test::Builder::Level = $Test::Builder::Level + 2;
+        return ok(0, "$comment - no match in tag contents (including CDATA)");
       }
     }
     else {
       my $got =  $node->toString;
       $got =~ s/^.*="(.*)"/$1/;
+      local $Test::Builder::Level = $Test::Builder::Level + 2;
       return $like_sub->(like $got, $regex, $comment);
     }
   }
@@ -182,12 +191,20 @@ Test::XML::Simple - easy testing for XML
 =head1 SYNOPSIS
 
   use Test::XML::Simple tests=>5;
+
   xml_valid $xml, "Is valid XML";
+  xml_valid $xml_doc_obj, "Is valid XML";
   xml_node $xml, "/xpath/expression", "specified xpath node is present";
   xml_is, $xml, '/xpath/expr', "expected value", "specified text present";
   xml_like, $xml, '/xpath/expr', qr/expected/, "regex text present";
   xml_is_deeply, $xml, '/xpath/expr', $xml2, "structure and contents match";
 
+  # XML::LibXML::Document can be passed as argument too
+  #  that allow you to test a big documents with several tests
+  my $xml_doc = XML::LibXML->createDocument( '1.0' );
+  xml_node $xml_doc, '/xpath/expression', 'specified xpath node is present';
+  xml_like, $xml_doc, '/xpath/expression', qr/expected result/, 'regex present';
+
   # Not yet implemented:
   # xml_like_deeply would be nice too...
 
@@ -196,6 +213,7 @@ Test::XML::Simple - easy testing for XML
 C<Test::XML::Simple> is a very basic class for testing XML. It uses the XPath
 syntax to locate nodes within the XML. You can also check all or part of the
 structure vs. an XML fragment.
+All routines accept as first argument string with XML or XML::LibXML::Document object.
 
 =head1 TEST ROUTINES
 
@@ -251,9 +269,9 @@ content. Uses Test::LongString's C<is_string> function to do the test.
 
 Joe McMahon, E<lt>mcmahon@cpan.orgE<gt>
 
-=head1 LEGAL
+=head1 LICENSE
 
-Copyright (c) 2005 by Yahoo!
+Copyright (c) 2005 by Yahoo! and Joe McMahon
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself, either Perl version 5.6.1 or, at
